import os
import logging


class Logger:
    def __init__(self):
        pass

    @classmethod
    def format_message(cls, message, logging_info=None):
        formatted_message = "{} {}"
        return formatted_message.format(logging_info if logging_info else "", message)

    @classmethod
    def info(cls, message, logging_info=None):
        message_to_print = cls.format_message(message, logging_info)
        if os.getenv('IS_DEV'):
            print(message_to_print)
        else:
            logging.info(message_to_print)

    @classmethod
    def error(cls, message, logging_info=None):
        message_to_print = cls.format_message(message, logging_info)
        if os.getenv('IS_DEV'):
            print(message_to_print)
        else:
            logging.error(message_to_print)

    @classmethod
    def warning(cls, message, logging_info=None):
        message_to_print = cls.format_message(message, logging_info)
        if os.getenv('IS_DEV'):
            print(message_to_print)
        else:
            logging.warning(message_to_print)




from microservices_common.exceptions import CustomException


class OwnerNotFound(CustomException):
    status_code = 400
    message = 'INVALID_INPUT'
    description = 'Failed to authenticate company, please try again later'

    def serialize_exception(self):
        return dict(message=self.message, description=self.description), self.status_code




from microservices_common.exceptions import CustomException


class UserIdNotFound(CustomException):
    status_code = 400
    message = 'INVALID_INPUT'
    description = 'Failed to authenticate user, please try again later'

    def serialize_exception(self):
        return dict(message=self.message, description=self.description), self.status_code



from datetime import datetime, timedelta
from dateutil.parser import parse
import pytz
from microservices_common.exceptions import IncorrectParameterFormat


class DateTimeUtil:

    @classmethod
    def to_date_tz(cls, s):
        format = '%Y-%m-%dT%H:%M:%S%z'
        return datetime.strptime(s, format)

    @classmethod
    def to_date_utc(cls, s):
        format = '%Y-%m-%dT%H:%M:%S'
        return datetime.strptime(s, format)

    @classmethod
    def to_date_utc_db(cls, s):
        splitted = s.split('.')
        format = '%Y-%m-%dT%H:%M:%S'
        return datetime.strptime(splitted[0], format)

    @classmethod
    def to_date(cls, s):
        try:
            return cls.to_date_tz(s)
        except:
            try:
                return cls.to_date_utc(s)
            except:
                return cls.to_date_utc_db(s)

    @classmethod
    def to_date_isoformat(cls, s):
        return cls.to_date(s).isoformat()

    @classmethod
    def parse_datetime(cls, iso_str):
        if iso_str and (' ' in iso_str):
            raise IncorrectParameterFormat(
                'Incorrect format of datetime: "%s". '
                'Whitespace was found, which means an error exists. '
                'Be sure that you properly url-encode character "+", '
                'by replacing it with "%%2B"' % iso_str)
        try:
            return (iso_str and parse(iso_str)) or None
        except ValueError:
            raise IncorrectParameterFormat(
                'Unable to parse datetime: "%s". Expected format is ISO-8601' % iso_str)

    @classmethod
    def convert_to_utc(cls, datetime_with_tzinfo):
        if not datetime_with_tzinfo:
            return None, None

        utcoffset = datetime_with_tzinfo.utcoffset() or timedelta(0)
        dt_in_utc = datetime_with_tzinfo - utcoffset
        dt_in_utc = dt_in_utc.replace(tzinfo=None)
        return dt_in_utc, utcoffset

    @classmethod
    def convert_to_local_iso_str(cls, datetime, timezone):
        return datetime.replace(tzinfo=pytz.utc).astimezone(pytz.timezone(timezone))

    @classmethod
    def convert_validation_datetime_to_utc(cls, _datetime):
        parsed_datetime = cls.parse_datetime(_datetime.isoformat())
        dt_in_utc, utcoffset = cls.convert_to_utc(parsed_datetime)

        return parsed_datetime, dt_in_utc, utcoffset

    @classmethod
    def convert_validation_datetime_to_utc_iso_str(cls, _datetime):
        parsed_datetime, dt_in_utc, utcoffset = cls.convert_validation_datetime_to_utc(
            _datetime)
        return dt_in_utc.isoformat()

    @classmethod
    def convert_str_to_utc(cls, _datetime):
        parsed_datetime = cls.parse_datetime(_datetime)
        dt_in_utc, utcoffset = cls.convert_to_utc(parsed_datetime)

        return dt_in_utc



from microservices_common.exceptions import CustomException
from microservices_common.utils.util import Util
from werkzeug.exceptions import HTTPException


def handle_exception(e):
    print('in handle_exception')
    print(e)
    if isinstance(e, CustomException):
        response, status_code = e.serialize_exception()
        for key in list(response.keys()):
            if response[key] is None or response[key] == '':
                del response[key]
        return response, status_code

    if isinstance(e, HTTPException) and hasattr(e, 'description') and hasattr(e, 'code'):
        return dict(message=e.description), e.code

    Util.print_traceback()

    import os
    if not os.getenv('IS_DEV'):
        from google.cloud import error_reporting
        client = error_reporting.Client()
        client.report_exception(e)

    return dict(message='SERVER_ERROR', description='Something went wrong'), 500



from setuptools import setup, find_packages


setup(
    name="microservices_common",
    version="0.0.71",
    author="Ali Zaidi",
    author_email="support@arrivy.com",
    description="",
    long_description="",
    url="https://github.com/arrivy-dev/microservices-python-common",
    packages=find_packages('src'),
    package_dir={'': 'src'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'google-cloud-error-reporting==1.5.2',
        'google-cloud-storage==1.43.0',
        'google-cloud-tasks==2.7.1',
        'flask==2.0.2',
        'python-dateutil==2.8.2',
        'pytz==2022.6'
    ]
)
